/*****************************************************************************/
/* This class is used to draw, paint a surface SDL and show surface in OpenGL*/
/*****************************************************************************/

/*
This file is part of Tuxworld.

    Tuxworld is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Tuxworld is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Tuxworld.  If not, see <http://www.gnu.org/licenses/>.

Ce fichier fait partie de Tuxworld.

	Tuxworld est un logiciel libre ; vous pouvez le redistribuer ou le
	modifier suivant les termes de la GNU General Public License telle
	que publiée par la Free Software Foundation ; soit la version 3 de
	la licence, soit (à votre gré) toute version ultérieure.

	Tuxworld est distribué dans l'espoir qu'il sera utile, mais SANS
	AUCUNE GARANTIE ; pas même la garantie implicite de
	COMMERCIABILISABILITÉ ni d'ADÉQUATION à UN OBJECTIF PARTICULIER.
	Consultez la GNU General Public License pour plus de détails.

	Vous devez avoir reçu une copie de la GNU General Public License
	en même temps que Tuxworld ; si ce n'est pas le cas,
	consultez <http://www.gnu.org/licenses>.
*/

#ifndef DRAW
#define DRAW

#include <math.h>
#include <vector>
#include <iostream>
#include <string>
#include "graphic.h"
#include "vector2d.h"
#include <SDL/SDL_image.h>
#include <SDL/SDL_opengl.h>

#define COLOR_BLACK(color) color.r = 0; color.g = 0; color.b = 0;
#define COLOR_WHITH(color) color.r = 255; color.g = 255; color.b = 255;
#define SIZE_TEXTURE_W_MAX 1024
#define SIZE_TEXTURE_H_MAX 512

class CDraw : public CGraphic
{ 

   public :

      // construct
      CDraw(const float& coef_resol_);
      // construct from a surface
      CDraw(SDL_Surface *img_, const SDL_Rect& coord_, const float& coef_resol_);
      // construct from a file
      CDraw(const std::string& file, const SDL_Rect& coord_, const float& coef_resol_);
      // free the surface
      void Free_surface();
      // destructor
      ~CDraw();
      // get pixel color
      SDL_Color get_pixel_color_SDL(const TVector2d<unsigned>& coord);
      // paint all surface with a color
      void full_color_SDL(const SDL_Color& color); 
      // clone image;
      SDL_Surface *copy_surface_SDL();
      // return image properties
      SDL_Rect get_properties();
      // return image in horizontal reverse
      void reverse_H_SDL();
      // copy src_surf surface
      SDL_Surface *copy_surface_SDL(SDL_Surface *src_surf);
      // blit surf on image or image on surf acordance with reverse value.
      void blit_SDL(const bool& reverse, SDL_Surface *surf, SDL_Rect *src, SDL_Rect dest);
      // we show the texture with opengl
      void show_GL(const float& deep);
      void show_GL(const bool& orientation, const SDL_Rect& dest, const float& deep);
      void show_GL(const bool& orientation, const SDL_Rect& dest, const TVector2d<float>& scale, const float& deep);
      void show_text_GL(const bool& orientation, const SDL_Rect& dest, const TVector2d<float>& text_coord, const float& deep);

      // we show the texture with opengl and a color
      void show_GL(const bool& orientation, const float& deep, const Color4f& color);
      void show_GL(const bool& orientation, const SDL_Rect& dest, const float& deep, const Color4f& color);

      // to scale a texture
      void scale(const TVector2d<float>& scale);

   protected :

      // texture for opengl
      int nbr_textures;
      GLuint *texture;
      GLenum texture_format;

   private :

      // calculate RGB or RGBA format for texture
      inline void texture_format_GL(void);
      // calculate the pow 2 near number i
      inline int pow2sup(int i);
      // to make a new image with width and height with pow 2
      SDL_Surface *destimg;

};

inline void CDraw::texture_format_GL(void) {

   // with alpha
   if(surface->format->BytesPerPixel == 4)
      if(surface->format->Rmask == 0x000000ff)
         texture_format = GL_RGBA;
      else
         texture_format = GL_BGRA;
   // no alpha channel
   else 
      if(surface->format->Rmask == 0x000000ff)
         texture_format = GL_RGB;
      else
         texture_format = GL_BGR;
}

inline int CDraw::pow2sup(int i) {
   double logbase2 = log(i) / log(2);
   return (int)floor(pow(2.0, ceil(logbase2)));
}

inline SDL_Surface *CDraw::copy_surface_SDL() {

   SDL_Surface *sdest = NULL;
   sdest = SDL_CreateRGBSurface(SDL_HWSURFACE, object_properties().w, object_properties().h, surface->format->BitsPerPixel, surface->format->Rmask, surface->format->Gmask, surface->format->Bmask, surface->format->Amask);
   sdest = NULL;
   return SDL_DisplayFormatAlpha(surface);

}

inline SDL_Surface *CDraw::copy_surface_SDL(SDL_Surface *src_surf) {

   SDL_Surface *sdest = NULL;
   sdest = SDL_CreateRGBSurface(SDL_HWSURFACE, src_surf->w, object_properties().h, src_surf->format->BitsPerPixel, src_surf->format->Rmask, src_surf->format->Gmask, src_surf->format->Bmask, src_surf->format->Amask);
   sdest = NULL;
   return SDL_DisplayFormatAlpha(src_surf);

}

inline void CDraw::show_GL(const float& deep) {
   show_GL(false, surface_properties(), deep);
}

inline void CDraw::show_GL(const bool& orientation, const float& deep, const Color4f& color) {

   show_GL(orientation, surface_properties(), deep, color);

}

inline void CDraw::show_GL(const bool& orientation, const SDL_Rect& dest, const float& deep, const Color4f& color) {

   glColor4f(color.r, color.g, color.b, color.a);
   show_GL(orientation, dest, deep);
   glColor4f(1, 1, 1, 1);

}

#endif
