#include "draw_digits.h"

using namespace std;

CDraw_Digits::CDraw_Digits(const string& file_, const SDL_Rect& coord_, const float& coef_resol_) : CDraw(file_, coord_, coef_resol_) {

   // we init offset and size of a letter image
   src.y = 0;
   src.h = surface->h;
   src.w = round((float)surface->w / 10.0f);

   // only texture is saved, not surface because we don't use tests on surface
   Free_surface();
}

void CDraw_Digits::show(const int& d, const float& deep) {

   int n = d, m;
   coord = object_properties();

   // we work only with n < 1000
   try {
      if(n > 999)
         throw string("draw_digits.cpp : this number is too big ");
      }
      catch(string caught) {
         cerr << caught << endl;
         throw;
      }

   if(n > 99) {
      // We work only with d, 0 <= d <= 9 : dxx, where dxx = n
      show_gl(n / 100, deep);
      // go for the next number between 0 and 9
      coord.x += src.w;
   }

   if(n > 9) {
      // We work only with d, 0 <= d <= 9 : ndx, where xdx = n
      m = n%100;
      n = m;
      show_gl(m / 10, deep);
      // go for the next number between 0 and 9
      coord.x += src.w;
   }

   // We work only with d, 0 <= d <= 9 : xxd, where xxd = n
   show_gl(n%10, deep);

}

void CDraw_Digits::show_gl(const int& d, const float& deep)
{

   try {
      if(d > 9)
         throw string("draw_digits.cpp : this digit is too big ");
   }
   catch(string caught) {
      cerr << caught << endl;
      throw;
   }

   float gl_x = (float)coord.x * coef_resol;
   float gl_y = (float)coord.y * coef_resol;
   float gl_w = src.w * coef_resol;
   float gl_h = (float)src.h * coef_resol;

   // locate the right surface on all texture, .1 = 1 / 10. 
   float text_x0 = d * .1;
   float text_x1 = d * .1 + .1;

   //  Bind the texture object
   glBindTexture( GL_TEXTURE_2D, texture[0] );

   glPushMatrix();
   glTranslatef(gl_x, SIZE_TEXTURE_H_MAX * coef_resol - gl_y, deep);
   glBegin(GL_QUADS);

   glTexCoord2f(text_x0, 0);
   glVertex3i(0, 0, -1);
   glTexCoord2f(text_x1, 0);
   glVertex3i(gl_w, 0, -1);
   glTexCoord2f(text_x1, 1);
   glVertex3i(gl_w, -gl_h, -1);
   glTexCoord2f(text_x0, 1);
   glVertex3i(0, -gl_h, -1);
   glEnd();
   glPopMatrix();

}
