#include "init_gameobjects.h"

using namespace std;

void init_game_objects(s_game &game, const int& nbr_tux_lifes, const int& nbr_tux_coins, const string& file_ennemy, vector<string>& Vfiles_seq_ennemy, const string& file_boss, const string& file_boss_injured, const unsigned& level) {

   try {

      // sky and earth
      game.rect_sky = origins;
      game.rect_earth = { X_SIZE_OUT, 0};
      game.rect_sky.h = game.screen->h / game.coef_resol;
      game.rect_sky.w = game.screen->w / game.coef_resol;
      game.rect_earth.h = game.screen->h / game.coef_resol;
      game.rect_earth.w = game.screen->w / game.coef_resol;

      // init coordinates for objects (x, y)
      game.coord_hearts = {10, 10};
      game.coord_power = {256, 10};
      game.rect_text_score = { 128, 16 };
      game.rect_text_clock = { 808, 16 };
      game.rect_clock = { 878, 16 };
      game.rect_score = { 228, 16 };

      // for texts
      COLOR_BLACK(game.color_text);
      string str_score("score :");
      string str_clock("time :");

      // create score text surface
      game.text_surface_score = TTF_RenderText_Blended(game.font, str_score.c_str(), game.color_text);
      game.ODraw_text_score = new CDraw(game.text_surface_score, game.rect_text_score, game.coef_resol);
      // only texture is saved, not surface because we don't use tests on surface
      game.ODraw_text_score->Free_surface();

      game.rect_text_score.w = game.ODraw_text_score->object_properties().w;
      game.rect_text_score.h = game.ODraw_text_score->object_properties().h;

      // create score digits
      game.ODraw_digits_score = new CDraw_Digits(IMG_PATH + "digits/digits.png", game.rect_score, game.coef_resol);

      // create clock text surface
      game.text_surface_clock = TTF_RenderText_Blended(game.font, str_clock.c_str(), game.color_text);
      game.ODraw_text_clock = new CDraw(game.text_surface_clock, game.rect_text_clock, game.coef_resol);
      // only texture is saved, not surface because we don't use tests on surface
      game.ODraw_text_clock->Free_surface();

      game.rect_text_clock.w = game.ODraw_text_clock->object_properties().w;
      game.rect_text_clock.h = game.ODraw_text_clock->object_properties().h;

      // create clock digits
      game.ODraw_digits_clock = new CDraw_Digits(IMG_PATH + "digits/digits.png", game.rect_clock, game.coef_resol);

      // vectors : p for the position, v for the velocity, a for accelerate and w for the weight

      SDyn_obj dyn_obj = {
         TVector2d<float> (200.0f, 160.0f),
         TVector2d<float> (-2.0f, 0.0f),
         TVector2d<float> (0.0f, 0.0f),
         TVector2d<float> (0.0f, .9f)
      };

      // init dynamic tux
      game.OTux = new CDynamic_Tux(dyn_obj, nbr_tux_lifes, 1);
      game.OCalculator_tux = new CDynamic(*game.OTux);
      dyn_obj.v.x = -.5f;
      dyn_obj.w.y = .4f;

      // init dynamic fires
      for(unsigned i = 0; i < NBR_FIRES; ++i) {
         shared_ptr<CDynamic_Fire> Fire = make_shared<CDynamic_Fire>(dyn_obj, 0);
         Fire->killall();
         game.VFire.push_back(Fire);
         shared_ptr<CDynamic> Calc_Fire = make_shared<CDynamic>(*(Fire.get()));
         game.VCalc_Fire.push_back(Calc_Fire);
      }

      if(level == 1)
         // init dynamic nutts
         for(unsigned i = 0; i < NBR_NUTTS; ++i) {
            shared_ptr<CDynamic_Fire> Nutts = make_shared<CDynamic_Fire>(dyn_obj, 0);
            Nutts->killall();
            game.VNutts.push_back(Nutts);
            shared_ptr<CDynamic> Calc_Nutts = make_shared<CDynamic>(*(Nutts.get()));
            game.VCalc_Nutts.push_back(Calc_Nutts);
         }
      else game.VCalc_Nutts.clear();

      // init dynamic boss
      dyn_obj = {
         TVector2d<float> (600.0f, Y_START_OBJECT),
         TVector2d<float> (0.0f, 0.0f),
         TVector2d<float> (0.0f, 0.0f),
         TVector2d<float> (0.0f, 2.2f)
      };

      switch(level) {
         case 0:
            game.OBoss = new CDynamic_Boss_Rabbit(dyn_obj, 4);
            //game.OBoss = new CDynamic_Boss_Rabbit(dyn_obj, 20);
         break;
         case 1:
            game.OBoss = new CDynamic_Boss_Ladybug(dyn_obj, 30);
         break;
         case 2:
            dyn_obj.w.y = .4f;
            game.OBoss = new CDynamic_Boss_Bee(dyn_obj, 20);
         break;
      }

      game.OCalculator_boss = new CDynamic(*game.OBoss);

      // We create fires images (animation object)
      game.anim_fire = new CAnimation(false, 0, 1, seq_fire, 1, IMG_PATH + string("fire/fire_"), origins, game.coef_resol);

      // We create nutts images (animation object)
      game.anim_nutts = new CAnimation(false, 0, 11, NULL, 11, IMG_PATH + string("nutts/nutts_"), origins, game.coef_resol);

      // We create hearts images (animation object)
      game.anim_heart = new CAnimation(false, 0, 1, seq_heart, 1, IMG_PATH + string("heart/heart_"), origins, game.coef_resol);

      // We create power images (animation object)
      game.anim_power = new CAnimation(false, 0, 1, seq_power, 1, IMG_PATH + string("energy/energy_"), origins, game.coef_resol);

      // We create explosion animation
      for(unsigned i = 0; i < NBR_EXPLOSIONS; ++i) {

         shared_ptr<CAnimation> Explosion = make_shared<CAnimation>(false, 0, 15, seq_explosion, 24, IMG_PATH + string("explosion/explosion_"), origins, game.coef_resol);
         game.VAnim_explosion.push_back(Explosion);

      }

      // We create coins images
      game.anim_coin = new CAnimation(false, 0, 15, seq_coin, 120, IMG_PATH + string("coin/coin_"), origins, game.coef_resol);

      // We create powers icons images
      game.anim_level = new CAnimation(false, 0, 4, NULL, 4, IMG_PATH + string("power/power_"), origins, game.coef_resol);

      // We create tux death animation
      game.anim_tux_death_right = new CAnimation(false, 0, 3, seq_tux_death, 16, IMG_PATH + string("tux_death/tux_death_"), origins, game.coef_resol);

      // We create tux and enemies animations
      game.anim_baby_tux_right = new CAnimation(false, 0, 7, seq_tux_game, 96, IMG_PATH + string("baby_tux/baby_tux_"), origins, game.coef_resol);
      game.anim_tux_right = new CAnimation(false, 0, 7, seq_tux_game, 96, IMG_PATH + string("tux/tux_"), origins, game.coef_resol);
      game.anim_supertux_right = new CAnimation(false, 0, 7, seq_tux_game, 96, IMG_PATH + string("supertux/supertux_"), origins, game.coef_resol);

      game.anim_ennemy_right = new CAnimation(false, 0, 7, seq_tux_game, 96, file_ennemy.c_str(), origins, game.coef_resol);

      // We create boss animation
      switch(level) {
         case 0 :
            game.anim_boss_static = new CAnimation(false, 0, 60, NULL, 60, string(file_boss), origins, game.coef_resol);
            game.anim_boss_static_injured = new CAnimation(false, 0, 60, NULL, 60, string(file_boss_injured), origins, game.coef_resol);
            game.anim_boss_jump = new CAnimation(false, 60, 30, NULL, 30, string(file_boss), origins, game.coef_resol);
            game.anim_boss_jump_injured = new CAnimation(false, 60, 30, NULL, 30, string(file_boss_injured), origins, game.coef_resol);
            game.anim_boss_run = new CAnimation(false, 90, 30, NULL, 30, string(file_boss), origins, game.coef_resol);
            game.anim_boss_run_injured = new CAnimation(false, 90, 30, NULL, 30, string(file_boss_injured), origins, game.coef_resol);
            game.anim_boss_shut = new CAnimation(false, 120, 20, NULL, 20, string(file_boss), origins, game.coef_resol);
            game.anim_boss_shut_injured = new CAnimation(false, 120, 20, NULL, 20, string(file_boss_injured), origins, game.coef_resol);
         break;
         case 1 :
            game.anim_boss_static = new CAnimation(false, 0, 40, NULL, 40, string(file_boss), origins, game.coef_resol);
            game.anim_boss_static_injured = new CAnimation(false, 0, 40, NULL, 40, string(file_boss_injured), origins, game.coef_resol);
            game.anim_boss_jump = NULL;
            game.anim_boss_jump_injured = NULL;
            game.anim_boss_shut = new CAnimation(false, 40, 40, NULL, 40, string(file_boss), origins, game.coef_resol);
            game.anim_boss_shut_injured = new CAnimation(false, 40, 40, NULL, 40, string(file_boss_injured), origins, game.coef_resol);
            game.anim_boss_run = new CAnimation(false, 80, 40, NULL, 40, string(file_boss), origins, game.coef_resol);
            game.anim_boss_run_injured = new CAnimation(false, 80, 40, NULL, 40, string(file_boss_injured), origins, game.coef_resol);
         break;
         case 2 :
            game.anim_boss_jump = NULL;
            game.anim_boss_jump_injured = NULL;
            game.anim_boss_run = new CAnimation(false, 0, 20, NULL, 20, string(file_boss), origins, game.coef_resol);
            game.anim_boss_run_injured = new CAnimation(false, 0, 20, NULL, 20, string(file_boss_injured), origins, game.coef_resol);
            game.anim_boss_static = new CAnimation(false, 20, 20, NULL, 20, string(file_boss), origins, game.coef_resol);
            game.anim_boss_static_injured = new CAnimation(false, 20, 20, NULL, 20, string(file_boss_injured), origins, game.coef_resol);
            game.anim_boss_shut = new CAnimation(false, 40, 10, NULL, 10, string(file_boss), origins, game.coef_resol);
            game.anim_boss_shut_injured = new CAnimation(false, 40, 10, NULL, 10, string(file_boss_injured), origins, game.coef_resol);
         break;
      }

      // create sequence coord for explosions
      for(unsigned i = 0; i < NBR_EXPLOSIONS; ++i)
         game.Vcoord_explosion.push_back(TVector2d<int>(seq_coord_explosions[i][0], seq_coord_explosions[i][1]));

      // create sequence coord where ennemies appear
      vector<shared_ptr<CDraw> > VSeq_draw_ennemies;
      SDL_Color test_color;

      for(vector<string>::iterator i = Vfiles_seq_ennemy.begin(); i != Vfiles_seq_ennemy.end(); ++i) {

         shared_ptr<CDraw> draw = make_shared<CDraw>(*i, origins, game.coef_resol);
         VSeq_draw_ennemies.push_back(draw);

      }

      // clear vectors
      game.VSeq_ennemies.clear();
      game.VCalc_Ennemy.clear();
      game.VEnnemy.clear();
      game.VSeq_coins.clear();
      game.VCalc_Coin.clear();
      game.VCoin.clear();
      game.VSeq_powers.clear();
      game.VCalc_Power.clear();
      game.VPower.clear();

      // We create resultants vectors and points position for the earth on y axis
      float ry_0 = 0.0, ry_1;

      float average = 0.0f;

      // size of draw without testing color
      unsigned t_x = 0;
      bool tested;

      int j = 0;
      for(vector<shared_ptr<CDraw> >::iterator i = VSeq_draw_ennemies.begin(); i != VSeq_draw_ennemies.end(); ++i) {

         // we start at 0, 0
         TVector2d<unsigned> coord(0, 0);

         // along width image
         for(; coord.x < i->get()->object_properties().w; ++coord.x) {

            if(t_x > 0)
               ++t_x;

            if(t_x > 10)
               t_x = 0;

            tested = false;

            // along height image
            for(coord.y = 0; coord.y < (*i)->object_properties().h; ++coord.y) {

               // We get the pixel color
               test_color = i->get()->get_pixel_color_SDL(coord);

               if(!tested && !t_x) {
                  // If it's a blue color, then it's an ennemy
                  if(test_color.r == 0 && test_color.g == 0 && test_color.b == 255) {
                     game.VSeq_ennemies.push_back(coord + TVector2d<unsigned> (j * (*i)->object_properties().w, 0));
                     ++t_x;
                     tested = true;
                  }

                  // If it's a red color, then it's a coin
                  if(test_color.r == 255 && test_color.g == 0 && test_color.b == 0) {
                     game.VSeq_coins.push_back(coord + TVector2d<unsigned> (j * (*i)->object_properties().w, 0));
                     ++t_x;
                     tested = true;
                  }

                  // If it's a green color, then it's a power item
                  if(test_color.g == 255 && test_color.r == 0 && test_color.b == 0) {
                     game.VSeq_powers.push_back(coord + TVector2d<unsigned> (j * (*i)->object_properties().w, 0));
                     ++t_x;
                     tested = true;
                  }
               }

               // If it's a black color, then it's the ground contact
               if(!(test_color.r | test_color.g | test_color.b)) {

                  ry_1 = (float)coord.y;
                  if(!coord.x)
                     ry_0 = ry_1;
                  break;
               }
            }
 
            // not ground impact ? Then ry_1 = 0 to indicate not ground impact.
            if(coord.y == (*i)->object_properties().h)
               ry_1 = 0.0f;

            game.resultants_y_earth.push_back((ry_1 - ry_0));
            // smoth vectors
            average += ry_1 - ry_0;
            if(!(coord.x%AVERAGE_MAX) && coord.x) {
               average /= AVERAGE_MAX;
               for(unsigned j = 0; j < AVERAGE_MAX; ++j)
                  game.resultants_y_earth.at( game.resultants_y_earth.size() - j - 1) = average * (AVERAGE_MAX - 1);
              
               average = 0.0f;
            }
            game.ry_earth.push_back(ry_1);
            ry_0 = ry_1;
         }
         ++j;
      }

      for(vector<shared_ptr<CDraw> >::iterator i = VSeq_draw_ennemies.begin(); i != VSeq_draw_ennemies.end(); ++i)
         // only texture is saved, not surface because we don't use tests on surface
         i->get()->Free_surface();

      VSeq_draw_ennemies.clear();
 
      game.nbr_ennemies_seq = game.VSeq_ennemies.size() / NBR_ENNEMIES;
      // Create ennemies vector
      dyn_obj.w.y = .4f;
      for(unsigned i = 0; i < NBR_ENNEMIES; ++i) {

         dyn_obj.p = *game.VSeq_ennemies.begin();

         shared_ptr<CDynamic_Ennemy> Ennemy = make_shared<CDynamic_Ennemy>(dyn_obj, 0);
         game.VEnnemy.push_back(Ennemy);
         shared_ptr<CDynamic> Calc_Ennemy = make_shared<CDynamic>(*(Ennemy.get()));
         game.VCalc_Ennemy.push_back(Calc_Ennemy);
         game.VSeq_ennemies.erase(game.VSeq_ennemies.begin());
      }

      // Create coins vector
      dyn_obj.w.y = .0f;
      for(unsigned i = 0; i < NBR_COINS; ++i) {

         dyn_obj.p = *game.VSeq_coins.begin();

         shared_ptr<CDynamic_Coin> Coin = make_shared<CDynamic_Coin>(dyn_obj, 0);
         game.VCoin.push_back(Coin);
         shared_ptr<CDynamic> Calc_Coin = make_shared<CDynamic>(*(Coin.get()));
         game.VCalc_Coin.push_back(Calc_Coin);
         game.VSeq_coins.erase(game.VSeq_coins.begin());
      }

      // Create powers vector
      for(unsigned i = 0; i < NBR_POWERS; ++i) {

         dyn_obj.p = *game.VSeq_powers.begin();
         shared_ptr<CDynamic_Coin> Power = make_shared<CDynamic_Coin>(dyn_obj, 0);
         game.VPower.push_back(Power);
         shared_ptr<CDynamic> Calc_Power = make_shared<CDynamic>(*(Power.get()));
         game.VCalc_Power.push_back(Calc_Power);
         game.VSeq_powers.erase(game.VSeq_powers.begin());
      }

   }
 
   catch(string &caught) {
      cerr << "init_gameobjects.cpp : Can't build objects." << endl;
      exit_game(game);
   }
}
