#include "level_game.h"

using namespace std;

inline void show_boss(CAnimation *anim, CAnimation *anim_pass, CDynamic_Boss *Obr) {
   ANIM_MOVE_TO_CENTER(anim, Obr);
   if(Obr->orientation_read())
      anim->show(LOOP, false, 1, .2f);
   else
      anim->show(LOOP, true, 1, .2f);
   anim_pass->pass(true, 1);
}

inline void update_scene() {
   glFlush();
   SDL_GL_SwapBuffers();
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
}

inline void show_sky_ground(s_game& game) {

   if(!game.OTux->death_read()) {
      // when tux is near window sides, game.step_sky and step_ground are big
      float amp_step = .08 * (game.OTux->position_read().x - ( game.screen->w / (game.coef_resol * 2) ));

      // Stop, Tux can't go away in left
      if((game.rect_earth.x > X_SIZE_OUT || amp_step > .0f)
       // Stop, Tux can't go away in right
       && (game.rect_earth.x < game.VDrawW_ground.at(0)->object_properties().w * 2 - game.screen->w / game.coef_resol - X_SIZE_OUT || amp_step < .0f)) {

         // We eval new ground and sky images positions
         game.step_sky = STEP_SKY * amp_step;
         game.step_ground = round(STEP_EARTH * amp_step);

         game.rect_sky.x += game.step_sky;
         game.rect_earth.x += game.step_ground;

         if(game.rect_sky.x < 0) {
            game.rect_sky.x = 0;
            game.step_sky = 0;
         }
         if(game.rect_sky.x > game.ODrawW_sky->object_properties().w - game.screen->w / game.coef_resol - X_SIZE_OUT) {
            game.rect_sky.x = game.ODrawW_sky->object_properties().w - game.screen->w / game.coef_resol - X_SIZE_OUT;
            game.step_sky = 0;
         }

      }
      else {
         if((game.OTux->velocity_read().x < .0f && game.OTux->position_read().x < X_SIZE_OUT)
          ||(game.OTux->velocity_read().x > .0f && game.OTux->position_read().x > game.VDrawW_ground.at(0)->object_properties().w * 2 - game.screen->w / game.coef_resol - X_SIZE_OUT))
            game.OTux->accel(TVector2d<float>(-game.OTux->velocity_read().x, .0f));

         game.step_ground = 0;
         game.step_sky = 0;
      }
   }

   game.ODrawW_sky->show(game.rect_sky, 0, 0.0f);

   // Now we draw sky and ground
   if(game.rect_earth.x < game.VDrawW_ground.at(0)->object_properties().w + game.screen->w / game.coef_resol)
      game.VDrawW_ground.at(0)->show(game.rect_earth, 0, 0.1f);

   if(game.rect_earth.x >= game.VDrawW_ground.at(0)->object_properties().w - game.screen->w / game.coef_resol)
      game.VDrawW_ground.at(1)->show(game.rect_earth, game.VDrawW_ground.at(0)->object_properties().w, 0.1f);

}

// We move the tux animation and show it
void anim_tux(s_game& game, CAnimation *anim_tux_right) {

   ANIM_MOVE_TO_CENTER(anim_tux_right, game.OTux);
      
   switch(game.OTux->run_read()) {
      case( RUN_RIGHT_ON ) :
         anim_tux_right->show(LOOP, false, game.OTux->step_speed_read(), 1.0);
      break;
      case( RUN_RIGHT_OFF ) :
         anim_tux_right->show(NO_LOOP, false, game.OTux->step_speed_read(), 1.0);
      break;
      case( RUN_LEFT_ON ) :
         anim_tux_right->show(LOOP, true, game.OTux->step_speed_read(), 1.0);
      break;
      case( RUN_LEFT_OFF ) :
         anim_tux_right->show(NO_LOOP, true, game.OTux->step_speed_read(), 1.0);
   }
}

void level_game(s_game& game, const unsigned& nbr_appearances_enemies, const unsigned& level, const string& file_music, const string& file_ennemy, const string& file_sky, vector<string>& Vfiles_ground, vector<string>& Vfiles_seq_ennemy, const string& file_boss, const string& file_boss_injured) {

   // a : acceleration, p : position, v : velocity, w : weight
   SDyn_obj dyn_obj;

   // events keyboard or mouse
   SDL_Event event;

   // temporary id life value of an object
   int buff_id_life;

   // we init the game
   init_datas_world(game, file_music, file_sky, Vfiles_ground);
   init_game_objects(game, NBR_LIFES, 0, file_ennemy, Vfiles_seq_ennemy, file_boss, file_boss_injured, level);

   // init music
   game.sounds->play_music();


   // Main loop
   while(1) {
		
      SDL_Delay(TimeLeft());

      // update the screen
      update_scene();

      // show sky and ground
      show_sky_ground(game);

      // decreases the time remaining
      if(!game.clock_ticks) {
         --game.clock;
         if(!game.clock)
            game.OTux->death_eval();
      }

      // decreases the time every CLOCK_TICKS value
      game.clock_ticks ? --game.clock_ticks : game.clock_ticks = CLOCK_TICKS;

      // show the clock
      game.ODraw_text_clock->show_GL(false, game.ODraw_text_clock->surface_properties(), TVector2d<float>(2.0, 2.0), 0.3f);
      game.ODraw_digits_clock->show(game.clock, 0.3f);

      // We show the score
      game.ODraw_text_score->show_GL(false, game.ODraw_text_score->surface_properties(), TVector2d<float>(1.8, 2.0), 0.3f);
      game.ODraw_digits_score->show(game.OTux->nbr_coins_read(), 0.3f);

      if(game.OBoss->death_read()) {
         unsigned ticks_end = 0;

         // tux run away
         game.OTux->accel(TVector2d<float>(-game.OTux->velocity_read().x, .0f));
         game.OTux->run_eval(RUN_RIGHT_OFF);

         // Main loop
         while(1) {
		
            SDL_Delay(TimeLeft());

            // update the screen
            update_scene();

            // show sky and ground
            show_sky_ground(game);

            // check tux moves
            game.OCalculator_tux->check( RESULTANTS_Y_EARTH( round(game.OTux->position_read().x) ), LIMIT_GLUE, RY_EARTH ( round(game.OTux->position_read().x )), -game.step_ground);

            // We calculate the step velocity
            game.OTux->step_speed_eval( int(1.0 + abs(game.OTux->velocity_read().x) )); 

            // animate tux
            switch(game.OTux->level_read()) {
               case 0 :
                  anim_tux(game, game.anim_baby_tux_right);
               break;
               case 1 :
                  anim_tux(game, game.anim_tux_right);
               break;
               case 2 :
                  anim_tux(game, game.anim_supertux_right);
            }

            // boss don't move
            game.OBoss->accel(TVector2d<float>(-game.OBoss->velocity_read().x, -game.OBoss->velocity_read().y));

            // check the boss
            game.OCalculator_boss->check( RESULTANTS_Y_EARTH( round(game.OBoss->position_read().x) ), LIMIT_GLUE, RY_EARTH( round(game.OBoss->position_read().x)), -game.step_ground);

            // anim the boss
            ANIM_MOVE_TO_CENTER(game.anim_boss_static, game.OBoss);
            if(game.OBoss->orientation_read())
               game.anim_boss_static->show(UP_DOWN_LOOP, false, 1, .2f);
            else
               game.anim_boss_static->show(UP_DOWN_LOOP, true, 1, .2f);

            for(unsigned i = 0; i < NBR_EXPLOSIONS; ++i) {
               if(ticks_end > 50 + i * 5) {
                  game.VAnim_explosion.at(0).get()->object_move(game.Vcoord_explosion.at(i).x + game.OBoss->position_read().x, game.Vcoord_explosion.at(i).y + game.OBoss->position_read().y);
                  game.VAnim_explosion.at(0).get()->show(LOOP, false, 1, .3);
               }
            }

            // tux go away after explosions and restart explosions
            if(game.OTux->run_read() == RUN_RIGHT_OFF)
               if(ticks_end == 100) {
                  game.OTux->accel(TVector2d<float>(8.0f, .0f));
                  game.OTux->run_eval(RUN_RIGHT_ON);
                  ticks_end = 49;
               }

            // We can exit the game
            while(SDL_PollEvent(&event))
               if(event.type == SDL_KEYDOWN)
                  if(event.key.keysym.sym == SDLK_q)
                     exit_game(game);

            ++ticks_end;

            // end of level and boss death animations
            if(game.rect_earth.x > game.VDrawW_ground.at(0)->object_properties().w * 2 - game.screen->w / game.coef_resol - X_SIZE_OUT)
               return;
         }
      }
 
      // tux hurts an enemy ?

      if(!game.OTux->injured())
         for(vector<shared_ptr<CDynamic_Ennemy> >::iterator i = game.VEnnemy.begin(); i != game.VEnnemy.end() && !game.OTux->death_read(); ++i)
            if(!i->get()->death_read())
               IF_COLLISION(game.OTux->position_read(), i->get()->position_read(), COLLISION_TUX_ENEMY)
               {
                  game.sounds->play_sound(2, 6);

                  // tux death
                  if(!game.OTux->level_read())
                     game.OTux->death_eval();
                  //decrease tux's level
                  else {
                     game.OTux->update_level(false);
                     game.OTux->init_injure();
                     i->get()->killall();
                     i->get()->accel(TVector2d<float>(.0f, -9.0f));
               }
            }

      // tux hurts a nutts ?

      if(level == 1)
         if(!game.OTux->injured())
            for(vector<shared_ptr<CDynamic_Fire> >::iterator i = game.VNutts.begin(); i != game.VNutts.end() && !game.OTux->death_read(); ++i)
               if(i->get()->life_read())
                  IF_COLLISION(game.OTux->position_read(), i->get()->position_read(), COLLISION_TUX_NUTTS)
                  {
                     game.sounds->play_sound(2, 6);

                     // tux death
                     if(!game.OTux->level_read())
                        game.OTux->death_eval();
                     //decrease tux's level
                     else {
                        game.OTux->update_level(false);
                        game.OTux->init_injure();
                        i->get()->killall();
                        i->get()->accel(TVector2d<float>(.0f, -9.0f));
                  }
              }

      // tux hurts the boss ?

      if(game.OBoss->activated())
         if(!game.OTux->injured())
            IF_COLLISION(game.OTux->position_read(), game.OBoss->position_read(), COLLISION_TUX_BOSS)
            {
               game.sounds->play_sound(2, 6);

               // tux death
               if(!game.OTux->level_read())
                  game.OTux->death_eval();
               //decrease tux's level
               else {
                  game.OTux->update_level(false);
                  game.OTux->init_injure();
               }
            }

      // fire hurts an ennemy ?
      for(vector<shared_ptr<CDynamic_Ennemy> >::iterator i = game.VEnnemy.begin(); i != game.VEnnemy.end(); ++i)
         for(vector<shared_ptr<CDynamic_Fire> >::iterator j = game.VFire.begin(); j != game.VFire.end(); ++j)
            if(j->get()->life_read())

               IF_COLLISION(j->get()->position_read(), i->get()->position_read(), COLLISION_FIRE_ENEMY)
               {
                  game.sounds->play_sound(4, 3);
                  i->get()->killall();
                  i->get()->accel(TVector2d<float>(.0f, -9.0f));
                  j->get()->killall();
               }

      // check new posions for tux
      game.OTux->check();

      //is tux out of the screen ?
      if(game.OTux->position_read().x < -X_SIZE_OUT
      || game.OTux->position_read().x > X_SIZE_OUT + game.screen->w / game.coef_resol
      || game.OTux->position_read().y > Y_SIZE_OUT + game.screen->h / game.coef_resol)
         game.OTux->death_eval();

      // if tux is death
      if (game.OTux->death_read())
         tux_death(game, event, file_ennemy, Vfiles_seq_ennemy, file_boss, file_boss_injured, level);
      else {

         // if not contact between tux and earth  
         if( !RY_EARTH( round(game.OTux->position_read().x) )
             || RY_EARTH( round(game.OTux->position_read().x) ) - round(game.OTux->position_read().y) > 6 * game.OTux->level_speed()
             || round(game.OTux->position_read().y) - RY_EARTH( round(game.OTux->position_read().x)) > TOLERANCE_GRND) 
         {

            if(game.OTux->velocity_read().y < 0.0 )
               game.OTux->accel(TVector2d<float>(.0f, 2.5f));

            game.OCalculator_tux->check(0.0, LIMIT_GLUE, 0.0, -game.step_ground);

            if(game.OTux->jump_read() != STOP_JUMP)
               game.OTux->accel(TVector2d<float>(.0f, (ACCEL_JUMP_TUX - game.OTux->velocity_read().y) * game.OTux->level_speed()));
         }
         else {

            // we jump
            if(game.OTux->jump_read() == JUMP) {

               game.sounds->play_sound(0, 4);
               game.OTux->accel(TVector2d<float>(.0f, (ACCEL_JUMP_TUX - game.OTux->velocity_read().y) * game.OTux->level_speed()));
               game.OTux->jump_eval(NOT_JUMP);

            }
            // we are on the ground
            else {

               game.OCalculator_tux->check( RESULTANTS_Y_EARTH( round(game.OTux->position_read().x) ), LIMIT_GLUE, RY_EARTH ( round(game.OTux->position_read().x )), -game.step_ground);
            }
         }
      }


      // tux catch a coin ?
      for(vector<shared_ptr<CDynamic_Coin> >::iterator i = game.VCoin.begin(); i != game.VCoin.end(); ++i)
         // the coin was not captured
         if(!i->get()->catch_read())
            IF_COLLISION(game.OTux->position_read(), i->get()->position_read(), COLLISION_TUX_COIN)
            {
               game.sounds->play_sound(1, 2);
               i->get()->catch_update();
               game.OTux->nbr_coins_update();
            }

      // when a coin is now hidden then we create a new coin
      vector<shared_ptr<CDynamic_Coin> >::iterator i = game.VCoin.begin();

      if( 1 + i->get()->id_life_read() < NBR_APPEARANCES_COINS )
         if( i->get()->position_read().x < -X_SIZE_OUT && RIGHT_SCREEN_POSITION > game.VSeq_coins.begin()->x) {

               dyn_obj.p.x = game.VSeq_coins.begin()->x - game.rect_earth.x;
               dyn_obj.p.y = game.VSeq_coins.begin()->y;
               dyn_obj.v.x = .0f;
               dyn_obj.v.y = .0f;
               dyn_obj.w.x = .0f;
               dyn_obj.w.y = .0f;

               buff_id_life = i->get()->id_life_read();

               // erase the coin
               game.VCoin.erase(i);
               game.VCalc_Coin.erase(game.VCalc_Coin.begin());
               game.VSeq_coins.erase(game.VSeq_coins.begin());

               // create the new coin and put at end vector
               shared_ptr<CDynamic_Coin> Coin = make_shared<CDynamic_Coin>(dyn_obj, 1 + buff_id_life);
               game.VCoin.push_back(Coin);
               shared_ptr<CDynamic> Calc_Coin = make_shared<CDynamic>(*(Coin.get()));
               game.VCalc_Coin.push_back(Calc_Coin);

            }

      vector<shared_ptr<CDynamic_Coin> >::iterator it_power = game.VPower.begin();

      if(it_power != game.VPower.end()) {

         // We draw powers
         if(!it_power->get()->catch_read())
            if(it_power->get()->position_read().x < X_SIZE_OUT + game.screen->w / game.coef_resol
            && it_power->get()->position_read().x > -X_SIZE_OUT) {
               ANIM_MOVE_TO_CENTER(game.anim_level, it_power->get());
               game.anim_level->show(LOOP, false, 1, .2f);
            }

         // tux catch the power coin
         IF_COLLISION(game.OTux->position_read(), it_power->get()->position_read(), COLLISION_TUX_COIN)
         {
            game.sounds->play_sound(5, 2);
            game.OTux->update_level(true);
            game.VPower.erase(it_power);
            game.VCalc_Power.erase(game.VCalc_Power.begin());
         }
      }

      // we draw power bar
      game.anim_power->object_move(game.coord_power.x, game.coord_power.y);
      TVector2d<float> scale_power(game.OTux->power_read(), 1.0f);
      game.anim_power->show(NO_LOOP, false, 1, scale_power, .3);

      // check new coins positions
      for(vector<shared_ptr<CDynamic> >::iterator i = game.VCalc_Coin.begin(); i != game.VCalc_Coin.end(); ++i)
         i->get()->check(0.0f, LIMIT_GLUE, 0.0, -game.step_ground);

      for(vector<shared_ptr<CDynamic> >::iterator i = game.VCalc_Power.begin(); i != game.VCalc_Power.end(); ++i)
         i->get()->check(0.0f, LIMIT_GLUE, 0.0, -game.step_ground);

      // boss shut enemies
      if(level == 0)
         if(game.OBoss->status() == BOSS_SHUT) {

            // to start on the boss's mounth
            dyn_obj.p.x = game.OBoss->position_read().x;
            dyn_obj.p.y = game.OBoss->position_read().y - 120;

            if(game.OBoss->orientation_read()) {
               dyn_obj.p.x -= 70;
               dyn_obj.v.x = -4.5f;
            }
            else {
               dyn_obj.p.x += 70;
               dyn_obj.v.x = 4.5f;
            }

            dyn_obj.v.y = .0f;
            dyn_obj.w.x = .0f;
            dyn_obj.w.y = .4f;

            // we create enemies
            shared_ptr<CDynamic_Ennemy> Ennemy = make_shared<CDynamic_Ennemy>(dyn_obj, game.nbr_ennemies_seq - 1);
            game.VEnnemy.push_back(Ennemy);
            shared_ptr<CDynamic> Calc_Ennemy = make_shared<CDynamic>(*(Ennemy.get()));
            game.VCalc_Ennemy.push_back(Calc_Ennemy);
         }

      int j = 0;
      for(vector<shared_ptr<CDynamic_Ennemy> >::iterator i = game.VEnnemy.begin(); i != game.VEnnemy.end(); ++i) {
         // when can we kill the ennemie ?
         if(
            TRANS_POSITION(i->get()->position_read().x) > game.VDrawW_ground.at(0)->object_properties().w * 2
         || i->get()->position_read().x < -X_SIZE_OUT
         || i->get()->position_read().y > Y_SIZE_OUT + game.screen->h / game.coef_resol
         || TRANS_POSITION(i->get()->position_read().x) < 0
         ) {
            buff_id_life = i->get()->id_life_read();
            game.VEnnemy.erase(i);
            game.VCalc_Ennemy.erase(game.VCalc_Ennemy.begin() + j);

            if(game.OBoss->status() != BOSS_SHUT) {
               // too ennemies, we don't create more
               if(1 + buff_id_life > game.nbr_ennemies_seq - 1)
                  break;
               if(j > game.VSeq_ennemies.size() - 1)
                  break;

                  dyn_obj.p.x = game.VSeq_ennemies.at(j).x - game.rect_earth.x;
                  dyn_obj.p.y = game.VSeq_ennemies.at(j).y;
                  dyn_obj.v.x = -.5f;
                  dyn_obj.v.y = .0f;
                  dyn_obj.w.x = .0f;
                  dyn_obj.w.y = .4f;
            }

            shared_ptr<CDynamic_Ennemy> Ennemy = make_shared<CDynamic_Ennemy>(dyn_obj, 1 + buff_id_life);
            game.VEnnemy.push_back(Ennemy);
            shared_ptr<CDynamic> Calc_Ennemy = make_shared<CDynamic>(*(Ennemy.get()));
            game.VCalc_Ennemy.push_back(Calc_Ennemy);
            game.VSeq_ennemies.erase(game.VSeq_ennemies.begin() + j);

            }
         ++j;
      }

      // check new positions for all enemies
      j = 0;
      for(vector<shared_ptr<CDynamic_Ennemy> >::iterator i = game.VEnnemy.begin(); i != game.VEnnemy.end(); ++i) {

         // ennemy is not on the ground
         if(
            i->get()->death_read()
         || !RY_EARTH( round(i->get()->position_read().x) )
         || RY_EARTH( round(i->get()->position_read().x) ) > round(i->get()->position_read().y)
         || round(i->get()->position_read().y) - RY_EARTH( round(i->get()->position_read().x)) > TOLERANCE_GRND )
            game.VCalc_Ennemy.at(j)->check(0.0, LIMIT_GLUE, 0.0, -game.step_ground);
         // on the ground
         else
            game.VCalc_Ennemy.at(j)->check( RESULTANTS_Y_EARTH( round(i->get()->position_read().x) ), LIMIT_GLUE, RY_EARTH( round(i->get()->position_read().x)), -game.step_ground);
         ++j;
      }

      // activate boss at end of the level
      if( !game.OBoss->activated() && game.rect_earth.x > X_ACTIVATE_BOSS)
         game.OBoss->activate();
 
      if(game.OBoss->activated()) {

         // boss can't go away from this zone
         if( (game.OBoss->orientation_read() && TRANS_POSITION(game.OBoss->position_read().x) < ZONE_BOSS_INF)
          || (!game.OBoss->orientation_read() && TRANS_POSITION(game.OBoss->position_read().x) > ZONE_BOSS_SUP)) {
            if(level != 2)
               game.OBoss->accel(TVector2d<float>(-game.OBoss->velocity_read().x, 0));
            else
               game.OBoss->accel(TVector2d<float>(-game.OBoss->velocity_read().x - 6.0f, 0));
         }

         // if the boss is not on the ground
         if(level != 2) {
            if(
               game.OBoss->death_read()
            || !RY_EARTH( round(game.OBoss->position_read().x ))
            || RY_EARTH( round(game.OBoss->position_read().x) ) > round(game.OBoss->position_read().y)
            || round(game.OBoss->position_read().y) - RY_EARTH( round(game.OBoss->position_read().x)) > TOLERANCE_GRND
            )
               game.OCalculator_boss->check(0.0f, LIMIT_GLUE, 0.0, -game.step_ground);
               // on the ground
            else {
               // ground amortization
               if(game.OBoss->velocity_read().y < 0.0 )
                  game.OBoss->accel(TVector2d<float>(-game.OBoss->velocity_read().x / 4, -game.OBoss->velocity_read().y / 4));

            // check new position for the boss 
            game.OCalculator_boss->check( RESULTANTS_Y_EARTH( round(game.OBoss->position_read().x) ), LIMIT_GLUE, RY_EARTH( round(game.OBoss->position_read().x)), -game.step_ground);
            }
         } else {
            game.OCalculator_boss->check(0.0f, 0.0, 0.0, -game.step_ground);
         }
      }

      // We draw hearts
      for(unsigned i = 0; i < game.OTux->life_read(); ++i) {
         game.anim_heart->object_move(game.coord_hearts.x + STEP_HEARTS * i, game.coord_hearts.y);
         game.anim_heart->show(NO_LOOP, false, 1, .3);

      }

      // Fires
      j=0;
      for(vector<shared_ptr<CDynamic_Fire> >::iterator i = game.VFire.begin(); i != game.VFire.end(); ++i) {
         if(i->get()->life_read()) {

            // kill the fire when it's out
            if(i->get()->position_read().x < -X_SIZE_OUT
            || i->get()->position_read().x > X_SIZE_OUT + game.screen->w / game.coef_resol
            || i->get()->position_read().y > Y_SIZE_OUT + game.screen->h / game.coef_resol) {
               i->get()->killall();
               break;
            }

            // We draw the fire
            ANIM_MOVE_TO_CENTER(game.anim_fire, i->get());
            game.anim_fire->show(LOOP, false, 1, .4f);

            // increment fire's life
            i->get()->life_eval();

            // if fire is not on the ground
            if(
               !RY_EARTH( round(i->get()->position_read().x) )
            || RY_EARTH( round(i->get()->position_read().x) ) > round(i->get()->position_read().y)
            || round(i->get()->position_read().y) - RY_EARTH( round(i->get()->position_read().x)) > 40 ) 
               game.VCalc_Fire.at(j)->check(0, 2.0, 0.0, -game.step_ground);
            // on the ground
            else
               game.VCalc_Fire.at(j)->check( RESULTANTS_Y_EARTH( round(i->get()->position_read().x) ), 6.0, RY_EARTH( round(i->get()->position_read().x)), -game.step_ground);
         }
      j++;
      }

      // Nutts
      if(level == 1) {
         int j = 0;
         for(vector<shared_ptr<CDynamic_Fire> >::iterator i = game.VNutts.begin(); i != game.VNutts.end(); ++i) {
            if(i->get()->life_read()) {

               // kill the nutts when it's out
               if(i->get()->position_read().x < -X_SIZE_OUT
               || i->get()->position_read().x > X_SIZE_OUT + game.screen->w / game.coef_resol
               || i->get()->position_read().y > Y_SIZE_OUT + game.screen->h / game.coef_resol) {
                  i->get()->killall();
                  break;
               }

               // We draw the nutts
               ANIM_MOVE_TO_CENTER(game.anim_nutts, i->get());
               game.anim_nutts->show(LOOP, false, 1, .4f);

               // increment nutts's life
               i->get()->life_eval();

               // if nutts is not on the ground
               if(
               !RY_EARTH( round(i->get()->position_read().x) )
               || RY_EARTH( round(i->get()->position_read().x) ) > round(i->get()->position_read().y)
               || round(i->get()->position_read().y) - RY_EARTH( round(i->get()->position_read().x)) > 40 ) 
                  game.VCalc_Nutts.at(j)->check(0, 2.0, 0.0, -game.step_ground);
               // on the ground
               else
                  game.VCalc_Nutts.at(j)->check( RESULTANTS_Y_EARTH( round(i->get()->position_read().x) ), 6.0, RY_EARTH( round(i->get()->position_read().x)), -game.step_ground);
            }
         j++;
         }
      }

      // We draw coins
      for(vector<shared_ptr<CDynamic_Coin> >::iterator i = game.VCoin.begin(); i != game.VCoin.end(); ++i)
         if(!i->get()->catch_read()) {
            ANIM_MOVE_TO_CENTER(game.anim_coin, i->get());
            game.anim_coin->show(LOOP, false, 1, .2f);
         }

      j = 0;
      for(vector<shared_ptr<CDynamic_Ennemy> >::iterator i = game.VEnnemy.begin(); i != game.VEnnemy.end(); ++i) {

            // We show the animation enemy and change the velocity
            if(i->get()->orientation_read()) {
               ANIM_MOVE_TO_CENTER(game.anim_ennemy_right, i->get());
               game.anim_ennemy_right->show(LOOP, false, 1, .4f + (float)j*.1);
            }
            else {
               ANIM_MOVE_TO_CENTER(game.anim_ennemy_right, i->get());
               game.anim_ennemy_right->show(LOOP, true, 1, .4f + (float)j*.1);
            }

            // check new position for enemies
            i->get()->check();
         ++j;
      }

      // We eval the Tux's power
      game.OTux->power_eval(false);

      if(game.OBoss->activated()) {

         // AI of boss
         // check new Boss's actions
         game.OBoss->check(game.VFire, game.OTux->position_read());

         if(!game.OBoss->blink_injured_read()) {

            // We show the animation boss
            switch(game.OBoss->status()) {
               case BOSS_JUMP :
                  show_boss(game.anim_boss_jump, game.anim_boss_jump_injured, game.OBoss);
               break;
               case BOSS_RUN :
                  show_boss(game.anim_boss_run, game.anim_boss_run_injured, game.OBoss);
               break;
               case BOSS_ATTACK :
                  show_boss(game.anim_boss_shut, game.anim_boss_shut_injured, game.OBoss);
               break;
               default :
                  show_boss(game.anim_boss_static, game.anim_boss_static_injured, game.OBoss);
            }
         }
         else {
            game.sounds->play_sound(4, 3);
            
            // We show the animation boss
            switch(game.OBoss->status()) {
               case BOSS_JUMP :
                  show_boss(game.anim_boss_jump_injured, game.anim_boss_jump, game.OBoss);
               break;
               case BOSS_RUN :
                  show_boss(game.anim_boss_run_injured, game.anim_boss_run, game.OBoss);
               break;
               case BOSS_ATTACK :
                  show_boss(game.anim_boss_shut_injured, game.anim_boss_shut, game.OBoss);
               break;
               default :
                  show_boss(game.anim_boss_static_injured, game.anim_boss_static, game.OBoss);
            }
         }
      }

      // fire eval
      j = 0;
      if(game.OTux->fire_read() && game.OTux->power_read() > POWER_INF)
         for(vector<shared_ptr<CDynamic_Fire> >::iterator i = game.VFire.begin(); i != game.VFire.end() && j < 3; ++i)
            if(!i->get()->life_read()) {
               game.sounds->play_sound(3, 5);
               if(!j)
                  game.OTux->power_eval(true);
 
               // We delete the death fire
               game.VFire.erase(i);
               game.VCalc_Fire.erase(game.VCalc_Fire.begin()+j);
               
               // We create new fire
               dyn_obj.p = game.OTux->position_read();
               if(game.OTux->run_read() & (RUN_RIGHT_ON | RUN_RIGHT_OFF & MASK_LEFT))
                  dyn_obj.v.x = 20.0f + j * 3.0f;
               else
                  dyn_obj.v.x = -20.0f - j * 3.0f;
               dyn_obj.v.y = -3.0f * j * 2;
               dyn_obj.w.x = 0.0f;
               dyn_obj.w.y = 2.0f / exp(j / 2);
               dyn_obj.a.y = .0f;

               shared_ptr<CDynamic_Fire> df = make_shared<CDynamic_Fire>(dyn_obj, 60);
               game.VFire.push_back(df);
               shared_ptr<CDynamic> cf = make_shared<CDynamic>(*(df.get()));
               game.VCalc_Fire.push_back(cf);
               ++j;

               // If tux is level 2, it can shut 3 fires
               if(game.OTux->level_read() < 2)
                  break;
            }

      if(level == 1)
         if(game.OBoss->status() == BOSS_SHUT)
            for(vector<shared_ptr<CDynamic_Fire> >::iterator i = game.VNutts.begin(); i != game.VNutts.end(); ++i)
               if(!i->get()->life_read()) {

                  game.sounds->play_sound(3, 5);
 
                  // We delete the death nutts
                  game.VNutts.erase(i);
                  game.VCalc_Nutts.erase(game.VCalc_Nutts.begin());
               
                  // We create new nutts
                  dyn_obj.p = game.OBoss->position_read();
                  if(game.OBoss->orientation_read()) {
                     dyn_obj.p.x += -game.anim_boss_static->object_properties().w / 2;
                     dyn_obj.v.x = -15.0f;
                  }
                  else {
                     dyn_obj.p.x += game.anim_boss_static->object_properties().w / 2;
                     dyn_obj.v.x = 15.0f;
                  }
                  dyn_obj.v.y = -12.0f;
                  dyn_obj.w.x = 0.0f;
                  dyn_obj.w.y = 2.0f;
                  dyn_obj.a.y = .0f;

                  shared_ptr<CDynamic_Fire> df = make_shared<CDynamic_Fire>(dyn_obj, 50);
                  game.VNutts.push_back(df);
                  shared_ptr<CDynamic> cf = make_shared<CDynamic>(*(df.get()));
                  game.VCalc_Nutts.push_back(cf);
                  break;
               }

      // nutts eval

      // if tux is death, next lines are not for it
      if(game.OTux->death_read())
         continue;

      // anim tux
      switch(game.OTux->level_read()) {
         case 0:
            anim_tux(game, game.anim_baby_tux_right);
         break;
         case 1:
            anim_tux(game, game.anim_tux_right);
         break;
         case 2:
            anim_tux(game, game.anim_supertux_right);
      }

      // we change tux velocity
      switch(game.OTux->run_read()) {

         case RUN_RIGHT_ON :
            game.OTux->accel(TVector2d<float>(.8f * game.OJoystick->joymotion_read(), .0f));
         break;
         case RUN_LEFT_ON :
            game.OTux->accel(TVector2d<float>(-.8f * game.OJoystick->joymotion_read(), .0f));
         break;
         case RUN_RIGHT_OFF :
         case RUN_LEFT_OFF :
            if(game.OTux->velocity_read().x < -1.0)
               game.OTux->accel(TVector2d<float>(.5f, .0f));

            if(game.OTux->velocity_read().x > 1.0)
               game.OTux->accel(TVector2d<float>(-.5f, .0f));

            if(game.OTux->velocity_read().x >= -1.0 && game.OTux->velocity_read().x <= 1.0)
               game.OTux->accel(TVector2d<float>(-game.OTux->velocity_read().x, .0f));

      }

      // to limit the velocity
      float tux_velocity_max = game.OTux->level_speed() * TUX_VELOCITY_MAX;

      if(game.OTux->velocity_read().x > tux_velocity_max)
         game.OTux->accel(TVector2d<float>(-game.OTux->velocity_read().x + tux_velocity_max, .0f));

      if(game.OTux->velocity_read().x < -tux_velocity_max)
         game.OTux->accel(TVector2d<float>(-game.OTux->velocity_read().x - tux_velocity_max, .0f));

      // We calculate the step velocity
      game.OTux->step_speed_eval( int(1.0 + abs(game.OTux->velocity_read().x) )); 

      if(game.rect_earth.x == 0 && game.OTux->position_read().x < 200)
         game.OTux->accel(TVector2d<float>(2 - game.OTux->velocity_read().x, .0f));

      if(game.rect_earth.x > game.VDrawW_ground.at(0)->object_properties().w * 2 - game.screen->w / game.coef_resol - X_SIZE_OUT)
         game.OTux->accel(TVector2d<float>(-2 - game.OTux->velocity_read().x, .0f));


      // events gestion
      while(SDL_PollEvent(&event)) {

         switch(event.type) {

            case SDL_JOYAXISMOTION :

               if( ( event.jaxis.value > -JAXIS ) && ( event.jaxis.value < JAXIS ) ) {
                  if(game.OTux->run_read() == RUN_RIGHT_ON) 
                     game.OTux->run_eval(RUN_RIGHT_OFF);
                  if(game.OTux->run_read() == RUN_LEFT_ON) 
                     game.OTux->run_eval(RUN_LEFT_OFF);
               }

               if(event.jaxis.value > JAXIS)
                  game.OTux->run_eval(RUN_RIGHT_ON);
               if(event.jaxis.value < -JAXIS)
                  game.OTux->run_eval(RUN_LEFT_ON);

               game.OJoystick->joymotion_eval(event);
            break; 

            case SDL_JOYHATMOTION:
           
               if(event.jhat.value == SDL_HAT_CENTERED && (game.t_right == SDL_JOYHATMOTION || game.t_left == SDL_JOYHATMOTION)) { 
                  if(game.OTux->run_read() == RUN_RIGHT_ON) 
                     game.OTux->run_eval(RUN_RIGHT_OFF);
                  if(game.OTux->run_read() == RUN_LEFT_ON) 
                     game.OTux->run_eval(RUN_LEFT_OFF);
                  game.OJoystick->joymotion_reset();
               }

               if(event.jhat.value == SDL_HAT_CENTERED && game.t_jump == SDL_JOYHATMOTION)
                  if(game.OTux->jump_read() == JUMP)
                     game.OTux->jump_eval(STOP_JUMP);
 
               if(event.jhat.value == game.b_right && game.t_right == SDL_JOYHATMOTION) {
                  game.OTux->run_eval(RUN_RIGHT_ON);
                  game.OJoystick->joymotion_reset();
               }
                  
               if(event.jhat.value == game.b_left && game.t_left == SDL_JOYHATMOTION) {
                 game.OTux->run_eval(RUN_LEFT_ON);
                 game.OJoystick->joymotion_reset();
	       }

               if(event.jhat.value == game.b_fire && game.t_fire == SDL_JOYHATMOTION)
                  game.OTux->fire_eval();

               if(event.jhat.value == game.b_jump && game.t_jump == SDL_JOYHATMOTION)
                  game.OTux->jump_eval(JUMP);

            break;
            
            case SDL_JOYBUTTONDOWN:
               if(event.jbutton.button == game.b_right && game.t_right == SDL_JOYBUTTONDOWN) {
                  game.OTux->run_eval(RUN_RIGHT_ON);
                  game.OJoystick->joymotion_reset();
               }

               if(event.jbutton.button == game.b_left && game.t_left == SDL_JOYBUTTONDOWN) {
                  game.OTux->run_eval(RUN_LEFT_ON);
                  game.OJoystick->joymotion_reset();
               }

               if(event.jbutton.button == game.b_jump && game.t_jump == SDL_JOYBUTTONDOWN)
                  game.OTux->jump_eval(JUMP);

               if(event.jbutton.button == game.b_fire && game.t_fire == SDL_JOYBUTTONDOWN)
                     game.OTux->fire_eval();
            break;
            
            case SDL_JOYBUTTONUP:
               if(event.jbutton.button == game.b_right && game.t_right == SDL_JOYBUTTONDOWN) {
                  game.OTux->run_eval(RUN_RIGHT_OFF);
                  game.OJoystick->joymotion_reset();
               }

               if(event.jbutton.button == game.b_left && game.t_left == SDL_JOYBUTTONDOWN) {
                  game.OTux->run_eval(RUN_LEFT_OFF);
                  game.OJoystick->joymotion_reset();
               }

               if(event.jbutton.button == game.b_jump && game.t_jump == SDL_JOYBUTTONDOWN)
                  game.OTux->jump_eval(STOP_JUMP);
            break;
            
            case SDL_KEYDOWN:
               if(event.key.keysym.sym == SDLK_q)
                  exit_game(game);
               if(event.key.keysym.sym == game.b_fire)
                  game.OTux->fire_eval();
               if(event.key.keysym.sym == game.b_jump)
                  game.OTux->jump_eval(JUMP);
               if(event.key.keysym.sym == game.b_right) {
                  game.OTux->run_eval(RUN_RIGHT_ON);
                  game.OJoystick->joymotion_reset();
               }
               if(event.key.keysym.sym == game.b_left) {
                  game.OTux->run_eval(RUN_LEFT_ON);
                  game.OJoystick->joymotion_reset();
               }
            break;
            
            case SDL_KEYUP:
               if(event.key.keysym.sym == game.b_jump)
                  game.OTux->jump_eval(STOP_JUMP);
               if(event.key.keysym.sym == game.b_right) {
                  game.OTux->run_eval(RUN_RIGHT_OFF);
                  game.OJoystick->joymotion_reset();
               }
               if(event.key.keysym.sym == game.b_left) {
                  game.OTux->run_eval(RUN_LEFT_OFF);
                  game.OJoystick->joymotion_reset();
               }
         }
      }
   }
}
